﻿# coding: utf-8
import sys

"""
.. only:: ja

    OWLIFT Python Library のモジュール。

.. only:: en

    OWLIFT Python Library's module.
"""

from enum import Enum
from datetime import datetime
import numpy as np
import platform


class OwColor(Enum):
    """
    .. only:: ja

        カラーテーブルを表します。

        .. data:: GRAY

            Grayscale

        .. data:: BLUE_ORANGE

            Blue Orange

        .. data:: RAINBOW

            Rainbow

    .. only:: en

        Represents color tables.

        .. data:: GRAY

            Grayscale

        .. data:: BLUE_ORANGE

            Blue Orange

        .. data:: RAINBOW

            Rainbow

    """

    GRAY = 0
    BLUE_ORANGE = 1
    RAINBOW = 2

class OwGainControl(Enum):
    """
    .. only:: ja

        ゲイン制御の種類を表します。

        .. data:: MANUAL

            マニュアルゲイン制御

        .. data:: AUTO_LINEAR

            線形自動ゲイン制御

        .. data:: AUTO_NON_LINEAR

            非線形自動ゲイン制御

    .. only:: en

        Represents types of gain control.

        .. data:: MANUAL

            Manual gain control

        .. data:: AUTO_LINEAR

            Linear auto gain control

        .. data:: AUTO_NON_LINEAR

            Non-linear auto gain control
    """

    MANUAL = 0
    AUTO_LINEAR = 1
    AUTO_NON_LINEAR = 2


class OwWindowCorrectionType(Enum):
    """
    .. only:: ja

        保護窓使用時の補正の種類を表します。

        .. data:: NONE

            保護窓なし

        .. data:: GAT05_STD_HOUSING

            標準筐体 / GAT-05

    .. only:: en

        Represents a correction type for protection windows.

        .. data:: NONE

            No protection window

        .. data:: GAT05_STD_HOUSING

            Standard housing / GAT-05
    """
    NONE = 0
    GAT05_STD_HOUSING = 1


class OwTelemetryData:
    """
    .. only:: ja

        Telemetry Data を表します。

    .. only:: en

        Represents Telemetry Data.

    .. csv-table::
        :header: "Property", "Offset", "Description"
        :widths: 3, 1, 6

        telemetry_revision,0,"Telemetry Revision"
        time_counter,1,"Time Counter"
        status_bits,3,"Status Bits"
        status_ffc_complete,3,"Whether or not FFC state is completed which Status Bits represents"
        module_serial,5,"Module Serial #"
        software_revision,13,"Software Revision"
        frame_counter,20,"Frame Counter"
        frame_mean,22,"Frame Mean"
        fpa_temp_in_counts,23,"FPA Temp in counts"
        fpa_temp_kx100,24,"FPA Temp in Kelvin x 100"
        housing_temp_in_counts,25,"Housing Temp in counts"
        housing_temp_kx100,26,"Housing Temp in kelvin x 100"
        fpa_temp_ffc_kx100,29,"FPA Temp at last FFC"
        ffc_time_counter,30,"Time Counter at last FFC"
        housing_temp_ffc_kx100,32,"Housing Temp at last FFC"
        reserved_b0,0,"Reserved"
        emissivity_x8192,19,"Emissivity scaled by 8192"
        background_temp_kx100,20,"Background Temperature in kelvin x 100"
        atmospheric_trans_x8192,21,"Atmospheric Transmission scaled by 8192"
        atmospheric_temp_kx100,22,"Atmospheric Temperature in kelvin x 100"
        window_trans_x8192,23,"Window Transmission scaled by 8192"
        window_reflect_x8192,24,"Window Reflection scaled by 8192"
        window_temp_kx100,25,"Window Temperature in kelvin x 100"
        window_reflect_temp_kx100,26,"Window Reflected Temperature in kelvin x 100"
        gain_mode,5,"Gain Mode"
        effective_gain,6,"Effective Gain"
        gain_mode_desired_flag,7,"Gain Mode Desired Flag"
        temp_gain_mode_threshold_h_to_l_degc,8,"Temperature Gain Mode Threshold High to Low (degC)"
        temp_gain_mode_threshold_l_to_h_degc,9,"Temperature Gain Mode Threshold Low to High (degC)"
        temp_gain_mode_threshold_h_to_l_k,10,"Temperature Gain Mode Threshold High to Low (K)"
        temp_gain_mode_threshold_l_to_h_k,11,"Temperature Gain Mode Threshold Low to High (K)"
        population_gain_mode_threshold_h_to_l,14,"Population Gain Mode Threshold High to Low"
        population_gain_mode_threshold_l_to_t,15,"Population Gain Mode Threshold Low to High"
        gain_mode_roi,22,"Gain Mode ROI"
        t_linear_eabled_state,48,"TLinear Enabled State"
        t_linear_resolution,49,"TLinear Resolution"
        spot_meter_mean,50,"Spotmeter Mean"
        spot_meter_max,51,"Spotmeter Maximum"
        spot_meter_min,52,"Spotmeter Minimum"
        spot_meter_population,53,"Spotmeter Population"
        spot_meter_roi_start_row,54,"Spotmeter ROI Start Row"
        spot_meter_roi_start_col,55,"Spotmeter ROI Start Col"
        spot_meter_roi_end_row,56,"Spotmeter ROI End Row"
        spot_meter_roi_end_col,57,"Spotmeter ROI End Col"

    """

    _DTYPE_TELEM = [
        ("telemetry_revision",                      ">u2"),
        ("time_counter",                            "u4"),
        ("status_bits",                             "u4"),
        ("module_serial",                           "S16"),
        ("software_revision",                       ">u8"),
        ("reserved_a17",                            "S6"),
        ("frame_counter",                           ">u4"),
        ("frame_mean",                              ">u2"),
        ("fpa_temp_in_counts",                      ">u2"),
        ("fpa_temp_kx100",                          ">u2"),
        ("housing_temp_in_counts",                  ">u2"),
        ("housing_temp_kx100",                      ">u2"),
        ("reserved_a27",                            "S4"),
        ("fpa_temp_ffc_kx100",                      ">u2"),
        ("ffc_time_counter",                        "u4"),
        ("housing_temp_ffc_kx100",                  ">u2"),
        ("reserved_a33",                            "S94"),
        ("reserved_b0",                             "S38"),
        ("emissivity_x8192",                        ">u2"),
        ("background_temp_kx100",                   ">u2"),
        ("atmospheric_trans_x8192",                 ">u2"),
        ("atmospheric_temp_kx100",                  ">u2"),
        ("window_trans_x8192",                      ">u2"),
        ("window_reflect_x8192",                    ">u2"),
        ("window_temp_kx100",                       ">u2"),
        ("window_reflect_temp_kx100",               ">u2"),
        ("reserved_b27",                            "S106"),
        ("reserved_c0",                             "S10"),
        ("gain_mode",                               ">u2"),
        ("effective_gain",                          ">u2"),
        ("gain_mode_desired_flag",                  ">u2"),
        ("temp_gain_mode_threshold_h_to_l_degc",    ">u2"),
        ("temp_gain_mode_threshold_l_to_h_degc",    ">u2"),
        ("temp_gain_mode_threshold_h_to_l_k",       ">u2"),
        ("temp_gain_mode_threshold_l_to_h_k",       ">u2"),
        ("reserved_c12",                            "S4"),
        ("population_gain_mode_threshold_h_to_l",   ">u2"),
        ("population_gain_mode_threshold_l_to_t",   ">u2"),
        ("reserved_c16",                            "S12"),
        ("gain_mode_roi",                           ">u8"),
        ("reserved_c26",                            "S44"),
        ("t_linear_eabled_state",                   ">u2"),
        ("t_linear_resolution",                     ">u2"),
        ("spot_meter_mean",                         ">u2"),
        ("spot_meter_max",                          ">u2"),
        ("spot_meter_min",                          ">u2"),
        ("spot_meter_population",                   ">u2"),
        ("spot_meter_roi_start_row",                ">u2"),
        ("spot_meter_roi_start_col",                ">u2"),
        ("spot_meter_roi_end_row",                  ">u2"),
        ("spot_meter_roi_end_col",                  ">u2"),
        ("reserved_c58",                            "S44"),
        ("reserved_z",                              "S160"),
    ]

    def __init__(self, telem_bytes):
        self._ary = np.frombuffer(telem_bytes, self._DTYPE_TELEM)

    def __getattr__(self, name):
        try:
            return self._ary[name][0]
        except ValueError:
            return None

    @property
    def status_ffc_complete(self):
        return (self._c.status_bits >> 4) & 3 == 3

class OwFrameMeta:
    """
        .. only:: ja

            フレームに付随するメタデータを表します。

        .. only:: en

            Represents the meta data attached to a frame.
    """
    def __init__(self, telemetry_data, file_datetime, file_time_offset):
        self._telemetry_data = telemetry_data
        self._file_datetime = file_datetime
        self._file_time_offset = file_time_offset

    @property
    def telemetry_data(self):
        """
        .. only:: ja

            Telemetry Data。
            [読み取り専用]

        .. only:: en

            Telemetry data.
            [Read only]

        :Type:      :class:`OwTelemetryData`

        """
        return self._telemetry_data

    @property
    def file_datetime(self):
        """
        .. only:: ja

            Raw録画ファイル再生時のタイムスタンプ。
            [読み取り専用]

        .. only:: en

            Timestamp while playing a raw-recording file.
            [Read only]

        :Type:      datetime.datetime
        """
        return self._file_datetime

    @property
    def file_time_offset(self):
        """
        .. only:: ja

            Raw録画ファイル再生時の時間オフセット。
            [読み取り専用]

        .. only:: en

            Time offset while playing a raw-recording file.
            [Read only]

        :Type:      int
        """
        return self._file_time_offset

class OwStatus(Enum):
    """
    .. only:: ja

        エラーコードを表します。

        .. data:: OK

            成功。

        .. data:: INTERNAL_ERROR

            ライブラリ内部エラー。

        .. data:: NO_MEMORY

            メモリ不足。

        .. data:: DEVICE_NOT_FOUND

            デバイスが見つかりません。

        .. data:: DEVICE_NOT_READY

            デバイスの準備ができていません。

        .. data:: DEVICE_UNSUPPORTED

            サポートされていないデバイスです。

        .. data:: NOT_ENOUGH_SIZE

            領域のサイズが不足しています。

        .. data:: NOT_YET

            処理がまだ完了していません。

        .. data:: FIRMWARE_ERROR

            デバイスのファームウェアでエラーが発生しました。

        .. data:: SENSOR_ERROR

            デバイスのセンサでエラーが発生しました。

        .. data:: DIRECTSHOW_ERROR

            DirectShowがエラーコードを返しました。

        .. data:: V4L2_ERROR

            V4L2がエラーコードを返しました。

        .. data:: NULL_POINTER

            引数のポインタがNULLです。

        .. data:: INVALID_VALUE

            引数の値が不正です。

        .. data:: INVALID_STATE

            不正な状態で関数が実行されました。

        .. data:: FILE_IO_ERROR

            ファイルI/Oエラーが発生しました。

        .. data:: INVALID_FILE_FORMAT

            不正なファイルフォーマットです。

        .. data:: END_OF_FILE

            ファイルの終端です。

        .. data:: INCOMPATIBLE

            互換性がありません。

    .. only:: en

        Represents error codes.

        .. data:: OK

            Success.

        .. data:: INTERNAL_ERROR

            Library's internal error.

        .. data:: NO_MEMORY

            Not enough memory.

        .. data:: DEVICE_NOT_FOUND

            No device was found.

        .. data:: DEVICE_NOT_READY

            Device is not ready.

        .. data:: DEVICE_UNSUPPORTED

            Unsupported device.

        .. data:: NOT_ENOUGH_SIZE

            Not enough size of the area.

        .. data:: NOT_YET

            Processing is not completed.

        .. data:: FIRMWARE_ERROR

            Error occurred in device firmware.

        .. data:: SENSOR_ERROR

            Error occurred in device sensor.

        .. data:: DIRECTSHOW_ERROR

            Error occurred in DirectShow.

        .. data:: V4L2_ERROR

            Error occurred in V4L2.

        .. data:: NULL_POINTER

            Argument as a pointer type is NULL.

        .. data:: INVALID_VALUE

            Value of argument is invalid.

        .. data:: INVALID_STATE

            Function is called in invalid state.

        .. data:: FILE_IO_ERROR

            File I/O error occurred.

        .. data:: INVALID_FILE_FORMAT

            Invalid file format.

        .. data:: END_OF_FILE

            End of file.

        .. data:: INCOMPATIBLE

            Incompatible.
    """

    OK = 0
    INTERNAL_ERROR = 1
    NO_MEMORY = 2
    DEVICE_NOT_FOUND = 3
    DEVICE_NOT_READY = 4
    DEVICE_UNSUPPORTED = 5
    NOT_ENOUGH_SIZE = 6
    NOT_YET = 7
    FIRMWARE_ERROR = 10
    SENSOR_ERROR = 11
    DIRECTSHOW_ERROR = 12
    V4L2_ERROR = 13
    NULL_POINTER = 20
    INVALID_VALUE = 21
    INVALID_STATE = 22
    FILE_IO_ERROR = 23
    INVALID_FILE_FORMAT = 24
    END_OF_FILE = 25
    INCOMPATIBLE = 26


class OwException(Exception):
    """
    .. only:: ja

        例外を表します。

    .. only:: en

        Represents exception.
    """

    def __init__(self, value):
        super(OwException, self).__init__(value)

    @property
    def status(self):
        """
        .. only:: ja

            エラーの種類。
            [読み取り専用]

        .. only:: en

            The kind of error.
            [Read only]

        :Type:      :class:`OwStatus`

        """
        # set in the extension
        return OwStatus(int(self._status))

    @property
    def host_error(self):
        """
        .. only:: ja

            ホストエラー。
            [読み取り専用]

            ホストエラーは status が :data:`OwStatus.DIRECTSHOW_ERROR`
            または :data:`OwStatus.V4L2_ERROR` のときの、
            プラットフォームに依存したエラーコードです。

        .. only:: en

            Host error.
            [Read only]

            A host error is platform-dependant error code when 'status' is
            OwStatus.DIRECTSHOW_ERROR or OwStatus.V4L2_ERROR.

        :Type:      int
        """
        # set in the extension
        return self._host_error

from owlift._owlift_ext import \
    ow_get_lib_version, \
    ow_get_devices, \
    ow_release, \
    ow_get_product_id, \
    ow_get_firmware_version, \
    ow_get_serial_number, \
    ow_get_frame_size, \
    ow_set_decode_type, \
    ow_set_manual_gain_range, \
    ow_get_manual_gain_range, \
    ow_set_agc_range, \
    ow_get_agc_range, \
    ow_get_frame_counter, \
    ow_get_disconnected, \
    ow_get_frame, \
    ow_get_active, \
    ow_decode_raw, \
    ow_decode_temp, \
    ow_capture_start, \
    ow_capture_stop, \
    ow_set_frame_options, \
    ow_file_write_passive_start, \
    ow_file_write_passive_add, \
    ow_file_write_stop, \
    ow_file_read_passive_start, \
    ow_file_read_rewind, \
    ow_get_file_time_length, \
    ow_get_noise_filter, \
    ow_set_noise_filter, \
    ow_set_time_based_agc, \
    ow_get_time_based_agc, \
    ow_time_based_agc_setup, \
    ow_set_agc_roi_mask, \
    ow_set_window_correction, \
    ow_get_undistortion, \
    ow_set_undistortion, \
    ow_set_reflect_corr_file, \
    ow_clear_reflect_corr_file, \
    ow_reconnect, \
    ow_enable_high_gain_mode, \
    ow_run_ffc, \
    ow_command_run, \
    ow_command_get, \
    ow_command_set

def lib_version():
    """
    .. only:: ja

        ネイティブライブラリのバージョンを返します。

        :Return:        ネイティブライブラリのバージョン。

    .. only:: en

        Returns the native library's version number.

        :Return:        The native library's version number.

    :Return Type:   (int, int, int)
    """
    return ow_get_lib_version()


def _init():
    pass

def devices():
    """
    .. only:: ja

        デバイスの一覧を取得します。

        :Return:        デバイス一覧。

    .. only:: en

        Returns the list of devices.

        :Return:        The list of devices.

    :Return Type:   tuple of :class:`OwDevice`
    """
    _init()

    cobjs = ow_get_devices()
    devs = list()

    for cobj in cobjs:
        devs.append(OwDevice(cobj))

    return devs

def raw_play_start(file_path):
    """
    .. only:: ja

        Raw録画ファイルを再生します。

        * file_path (str)

            Raw録画ファイルのパス。

        :Return:
            Raw録画ファイルをデータソースとする仮想のデバイス。

    .. only:: en

        Play the raw-recording file.

        * file_path (str)

            The file path of a raw-recording file.

        :Return:
            The virtual device whose data source is the raw-recording file.

    :Return Type:   :class:`OwDevice`

    """
    _init()

    return OwDevice(ow_file_read_passive_start(file_path), raw_play = True)

class OwDevice(object):
    """
    .. only:: ja

        デバイスを表します。

    .. only:: en

        Represents the device.
    """

    _DTYPE_TEMP = np.dtype(np.float32)
    _DTYPE_RAW = np.dtype(np.uint16)
    _DTYPE_IMG = np.dtype(np.uint8)

    def __init__(self, cobj, raw_play = False):
        self._cobj = cobj
        self._color = OwColor.BLUE_ORANGE
        self._gain_control = OwGainControl.AUTO_LINEAR
        ow_set_decode_type(self._cobj, self._color.value, self._gain_control.value)
        self._frame_size = ow_get_frame_size(self._cobj)
        self._image_size = self._frame_size
        self._raw_play = raw_play
        self._raw_enabled = False
        self._image_enabled = True
        self._magnify_enabled = False
        self._upside_down_enabled = False
        self._rgb_order = True
        self._window_correction = OwWindowCorrectionType.NONE
        self._agc_roi_mask = None

        if raw_play:
            self._file_time_length = ow_get_file_time_length(self._cobj) / 1000
        else:
            self._file_time_length = 0

    def release(self):
        """
        .. only:: ja

            デバイスを解放します。

        .. only:: en

            Release the device.
        """
        ow_release(self._cobj)

    @property
    def product_id(self):
        """
        .. only:: ja

            デバイスのプロダクトID。
            [読み取り専用]

        .. only:: en

            Product ID of the device.
            [Read only]

        :Type:      int
        """
        return ow_get_product_id(self._cobj)

    @property
    def firmware_version(self):
        """
        .. only:: ja

            デバイスファームウェアのバージョン。
            [読み取り専用]

        .. only:: en

            Version number of the device firmware.
            [Read only]

        :Type:      int
        """
        return ow_get_firmware_version(self._cobj)

    @property
    def serial_number(self):
        """
        .. only:: ja

            赤外線センサのシリアル番号。
            [読み取り専用]

        .. only:: en

            Serial number of the LWIR sensor in the device.
            [Read only]

        :Type:      (int, int, int, int)
        """
        return ow_get_serial_number(self._cobj)

    @property
    def color(self):
        """
        .. only:: ja

            カラーテーブルの種類。
            [書き込み可]

        .. only:: en

            Type of color table.
            [Writable]

        :Type:      :class:`OwColor`
        """
        return self._color

    @color.setter
    def color(self, col):
        if not isinstance(col, OwColor):
            raise TypeError('color')
        ow_set_decode_type(self._cobj, col.value, self._gain_control.value)
        self._color = col

    @property
    def gain_control(self):
        """
        .. only:: ja

            ゲイン制御の種類。
            [書き込み可]

        .. only:: en

            Type of gain control.
            [Writable]

        :Type:      :class:`OwGainControl`

        """
        return self._gain_control

    @gain_control.setter
    def gain_control(self, gc):
        if not isinstance(gc, OwGainControl):
            raise TypeError
        ow_set_decode_type(self._cobj, self._color.value, gc.value)
        self._gain_control = gc

    @property
    def manual_gain_range(self):
        """
        .. only:: ja

            マニュアルゲイン制御の温度範囲(最低、最高)
            [書き込み可]

            最低温度未満の温度は最低温度の色に、最高温度以上の温度は最高温度の色に切り詰められます。

        .. only:: en

            Temperature range for Manual Gain Control (min, max).
            [Writable]

            The temperature less than the specified minimum temperature is corrected to
            the minimum color value.
            The temperature greater than the specified maximum temperature is corrected to
            the maximum color value.

        :Type:      (float, float)
        """
        return ow_get_manual_gain_range(self._cobj)

    @manual_gain_range.setter
    def manual_gain_range(self, range):
        ow_set_manual_gain_range(self._cobj, float(range[0]), float(range[1]))

    @property
    def agc_range(self):
        """
        .. only:: ja

            自動ゲイン制御の温度範囲(最低、最高)。
            [書き込み可]

            最低温度未満の温度は最低温度の色に、最高温度以上の温度は最高温度の色に切り詰められます。

        .. only:: en

            Temperature range for Auto Gain Control (min, max).
            [Writable]

            The temperature less than the specified minimum temperature is corrected to
            the minimum color value.
            The temperature greater than the specified maximum temperature is corrected to
            the maximum color value.

        :Type:      (float, float)
        """
        return ow_get_agc_range(self._cobj)

    @agc_range.setter
    def agc_range(self, range):
        ow_set_agc_range(self._cobj, float(range[0]), float(range[1]))

    @property
    def frame_size(self):
        """
        .. only:: ja

            フレームサイズ(幅、高さ)を取得します。
            [読み取り専用]

        .. only:: en

            Frame size (width, height).
            [Read only]

        :Type:      (int, int)

        """
        return self._frame_size

    @property
    def frame_counter(self):
        """
        .. only:: ja

            フレームカウンタ。
            [読み取り専用]

        .. only:: en

            Frame counter.
            [Read only]

        :Type:      int
        """
        return ow_get_frame_counter(self._cobj)

    @property
    def disconnected(self):
        """
        .. only:: ja

            デバイスが切断されたかどうか。
            [読み取り専用]

            Raw録画ファイル再生中のときは、ファイルの終端まで再生したかどうかを取得します。

        .. only:: en

            Whether or not the device was disconnected.
            [Read only]

            When playing raw-recording file, represents whether or not at the end of the file 

        :Type:      bool
        """
        return ow_get_disconnected(self._cobj)

    @property
    def alive(self):
        """
        .. only:: ja

            disconnected() と真偽逆の値。
            [読み取り専用]

        .. only:: en

            Reverse value of disconnected().
            [Read only]

        :Type:      bool
        """
        return not self.disconnected

    def _to_image(self, img_buf):
        if self._color == OwColor.GRAY and not self._magnify_enabled:
            img = np.frombuffer(img_buf, OwDevice._DTYPE_IMG) \
                .reshape(self._image_size[1], self._image_size[0])
        else:
            img = np.frombuffer(img_buf, OwDevice._DTYPE_IMG) \
                .reshape(self._image_size[1], self._image_size[0], 3)

        return img

    @property
    def frame(self):
        """
        .. only:: ja

            温度データ、画像データ、メタデータ のタプル。
            [読み取り専用]

            温度データ、画像データは幅x高さの次元を持つ配列です。
            温度データの各要素はケルビン値 (float) です。
            Raw録画ファイル再生中、ファイル終端に達したときは (None, None, None) を返します。

        .. only:: en

            Tuple of the the temperature data, the image data and the meta data.
            [Read only]

            The temperature data and the image data are arrays that have dimensions of width x height.
            The each element of the temperature data is a kelvin value (float).
            Returns (None, None, None) if EOF is reached while playing a raw-recording file.

        :Type:      (numpy.array, numpy.array, :class:`OwFrameMeta`)
        """
        temp_buf, img_buf, telem_bytes, ftimeofs, dt = ow_get_frame(self._cobj)

        if not temp_buf:
            return (None, None, None)

        temp = np.frombuffer(temp_buf, \
            OwDevice._DTYPE_RAW if self._raw_enabled else OwDevice._DTYPE_TEMP) \
            .reshape(self._frame_size[1], self._frame_size[0])
        if self._image_enabled:
            img = self._to_image(img_buf)
        else:
            img = None

        telem = OwTelemetryData(telem_bytes)

        if self._raw_play:
            fdtime = datetime(dt[0], dt[1], dt[2], dt[3], dt[4], dt[5], dt[6])
        else:
            fdtime = None

        return (temp, img, OwFrameMeta(telem, fdtime, ftimeofs))

    def raw_to_image(self, raw):
        """
        .. only:: ja

            Rawデータを画像に変換します。

            Rawデータは raw_enabled = True の状態で frame プロパティにより取得される形式と同じです。
            変換の過程において、設定に従ってノイズフィルタなどの処理が適用されます。
            この関数に渡したRawデータは raw_record_add() で処理される対象に置き換わります。
            すわなち frame プロパティ、raw_to_image()、raw_record_add() の順で実行すると
            Raw録画ファイルに追加されるフレームは raw_to_image() に渡したフレームとなります。

            * raw (numpy.array)

                Rawデータ。

            :Return:        画像を表す配列

        .. only:: en

            Converts the frame of raw data to a image.

            The format of raw data is same as one returning by frame property when raw_enabled is True.
            The processes such as the noise filter are applied following the settings.
            The raw data passed to this function replaces the target of raw_record_add().
            To be more specific, if you execute frame property, raw_to_image() and raw_record_add()
            in this order, the frame passed to raw_to_image() is added to a raw-recording file.

            * raw (numpy.array)

                Raw data.

            :Return:        The array that represents image.

        :Return Type:   numpy.array
        """

        return self._to_image(ow_decode_raw(self._cobj, raw))

    def temp_to_image(self, temp):
        """
        .. only:: ja

            温度データを画像に変換します。

            温度データは raw_enabled = False の状態で frame プロパティにより取得される形式と同じです。
            変換の過程において、ノイズフィルタなどの処理は適用されません。
            温度データを加工してから画像に変換したいときに利用します。

            :Return:        画像を表す配列

        .. only:: en

            Converts the frame of temperature data to a image.

            The format of temperature data is same as one returning by frame property when raw_enabled is False.
            The processes such as the noise filter are not applied.
            This is useful when you edit temperature data and convert to an image.

        :Return Type:   numpy.array

        """

        return self._to_image(ow_decode_temp(self._cobj, temp))

    def capture_start(self):
        """
        .. only:: ja

            デバイスからのデータ取得を開始します。

        .. only:: en

            Starts getting data from the device.

        """
        ow_capture_start(self._cobj)

    def capture_stop(self):
        """
        .. only:: ja

            デバイスからのデータ取得を停止します。

        .. only:: en

            Stops getting data from the device.
        """
        ow_capture_stop(self._cobj)

    @property
    def raw_enabled(self):
        """
        .. only:: ja

            frame プロパティで温度の代わりにRawデータを返すかどうか。
            [書き込み可]

        .. only:: en

            Whether or not returning raw data instead of temperature data by 'frame' property.
            [Writable]

        :Type:      bool
        """
        return self._raw_enabled

    def _update_frame_options(self):
        ow_set_frame_options(self._cobj, self._raw_enabled, self._image_enabled, \
            self._magnify_enabled, self._upside_down_enabled, self._rgb_order)

    @raw_enabled.setter
    def raw_enabled(self, enb):
        self._raw_enabled = enb
        self._update_frame_options()

    @property
    def image_enabled(self):
        """
        .. only:: ja

            frame プロパティで画像を返すかどうか。
            [書き込み可]

        .. only:: en

            Whether or not returning image by 'frame' property.
            [Writable]

        :Type:      bool
        """
        return self._raw_enabled

    @image_enabled.setter
    def image_enabled(self, enb):
        self._image_enabled = enb
        self._update_frame_options()

    @property
    def magnification_enabled(self):
        """
        .. only:: ja

            画像を縦横3倍に拡大するかどうか。
            [書き込み可]

        .. only:: en

            Whether or not magnifying the image 3 times vertically and horizontally.
            [Writable]

        :Type:      bool
        """
        return self._magnify_enabled

    @magnification_enabled.setter
    def magnification_enabled(self, enb):
        self._magnify_enabled = enb
        if enb:
            self._image_size = (self._frame_size[0] * 3, self._frame_size[1] * 3)
        else:
            self._image_size = self._frame_size
        self._update_frame_options()

    @property
    def upside_down_enabled(self):
        """
        .. only:: ja

            画像を180回転するかどうか。
            [書き込み可]

        .. only:: en

            Whether or not rotating the image 180 degrees.
            [Writable]

        :Type:      bool
        """
        return self._upside_down_enabled

    @upside_down_enabled.setter
    def upside_down_enabled(self, enb):
        self._upside_down_enabled = enb
        self._update_frame_options()

    @property
    def rgb_order(self):
        """
        .. only:: ja

            RGB形式のバイトオーダ。
            True: BGR, False: RGB
            [書き込み可]

        .. only:: en

            Byte-order for RGB format.
            True: BGR, False: RGB
            [Writable]

        :Type:      bool
        """
        return self._rgb_order

    @rgb_order.setter
    def rgb_order(self, bgr_or_rgb):
        self._rgb_order = bgr_or_rgb
        self._update_frame_options()

    def raw_play_rewind(self):
        """
        .. only:: ja

            Raw録画ファイル再生中に再生位置を先頭に戻します。

        .. only:: en

            Rewind the reading position while playing a raw-recording file.
        """
        ow_file_read_rewind(self._cobj)

    @property
    def file_time_length(self):
        """
        .. only:: ja

            Raw録画ファイルの長さ(秒単位)。
            [読み取り専用]

        .. only:: en

            Length of the raw-recording file in seconds.
            [Writable]

        :Type:      int
        """
        return self._file_time_length

    def raw_record_start(self, file_path, frame_rate_numerator = 3, frame_rate_denominator = 26):
        """
        .. only:: ja

            Raw録画を開始します。

            * file_path (str)

                保存先のファイルパス。

            * frame_rate_numerator (int)

                フレームレートの分子。

            * frame_rate_numerator (int)

                フレームレートの分母。

        .. only:: en

            Starts raw-recording.

            * file_path (str)

                The recording file path.

            * frame_rate_numerator (int)

                The numerator of the frame rate.

            * frame_rate_numerator (int)

                The denominator of the frame rate.

        """
        ow_file_write_passive_start(self._cobj, file_path, frame_rate_numerator, frame_rate_denominator)

    def raw_record_add(self):
        """
        .. only:: ja

            Raw録画ファイルへ最後に frame プロパティで取得したフレームデータを追加します。

        .. only:: en

            Adds the last frame data which was got by the 'frame' property to the raw-recording file.

        """
        ow_file_write_passive_add(self._cobj)

    def raw_record_stop(self):
        """
        .. only:: ja

            Raw録画を停止します。

        .. only:: en

            Stops raw-recording.

        """
        ow_file_write_stop(self._cobj)

    @property
    def noise_filter(self):
        """
        .. only:: ja

            ノイズフィルタの閾値。
            [書き込み可]

            閾値は温度差を意味しており、対象のピクセルとそれに近接したピクセルの温度差が
            指定の閾値を超えている場合、ノイズフィルタにより平均化されます。
            0 を指定するとノイズフィルタは無効になります。

        .. only:: en

            Threshold of the noise filter.
            [Writable]

            The threshold means integer value of temperature difference.
            If the difference of the target pixel and its adjacent pixels is over the threshold,
            the target pixel will be averaged by the noise filter.

        :Type:      float

        """
        return ow_get_noise_filter(self._cobj)

    @noise_filter.setter
    def noise_filter(self, value):
        ow_set_noise_filter(self._cobj, value)

    @property
    def undistortion(self):
        """
        .. only:: ja

            歪み補正が有効かどうか。
            [書き込み可]

            デフォルト値は OWLIFT Type-A/B が False, OWLIFT Type-F が True です。

        .. only:: en

            Whether or not Undistortion is enabled.
            [Writable]

            The default value is False for OWLIFT Type-A/B, is True for OWLIFT Type-F.

        :Type:      bool
        """
        return ow_get_undistortion(self._cobj)

    @undistortion.setter
    def undistortion(self, enb):
        ow_set_undistortion(self._cobj, enb)

    @property
    def raw_play_time_based_agc(self):
        """
        .. only:: ja

            Time-Based AGCが有効かどうか。
            [書き込み可]

            :func:`raw_play_start` により取得した :class:`OwDevice` オブジェクトに対してのみ使用可能です。
        .. only:: en

            Whether or not Time-Based AGC is enabled.
            [Writable]

            This function can be called against an :class:`OwDevice`  object
            that is got by func:`raw_play_start`.

        :Type:      bool
        """
        return ow_get_time_based_agc(self._cobj)

    @raw_play_time_based_agc.setter
    def raw_play_time_based_agc(self, enb):
        ow_set_time_based_agc(self._cobj, enb)

    def raw_play_time_based_agc_setup(self, frame_num_intvl = 10, from_time_offset = 0, to_time_offset = 0):
        """
        .. only:: ja

            Raw録画ファイルから一定間隔でフレームを抽出して、
            抽出したフレームを元に Time-Based AGC のセットアップを行います。

            * frame_num_intvl (int)

                フレームを抽出する間隔(何フレーム毎に抽出するか)。
            * from_time_offset (int)

                計算を開始するオフセット時間。

            * to_time_offset (int)

                計算を開始するオフセット時間。

        .. only:: en

            Extracts frames from the raw-recording file at the specified interval and
            setup Time-Based AGC  minimum and maximum temperatures of all the frames.

            * frame_num_intvl (int)

                The interval number of frames for extracting.

            * from_time_offset (int)

                The time offset to start calculating.

            * to_time_offset (int)

                The time offset to stop calculating.

        """
        ow_time_based_agc_setup(self._cobj, frame_num_intvl, from_time_offset, to_time_offset)

    @property
    def agc_roi_mask(self):
        """
        .. only:: ja

            自動ゲイン制御のROI(Region of Interest)。
            [書き込み可]

            興味のない領域にある高温または低温のピクセルに引きずられて全体の色合いが平坦になるときに利用します。
            値はシェイプが (フレーム高さ, フレーム幅)、型が bool か int の2次元行列です。
            ピクセルの位置に対応する要素が True または 0 以外のとき、そのピクセルが計算対象になります。
            全ての要素が False または 0 の行列を設定しようとすると例外が発生します。

        .. only:: en

            ROI (Region of Interest) of Auto Gain Control.
            [Writable]

            ROI is useful when colors look flat
            because there are pixels of high or low temperature at uninsteresting area.
            When the mask value that position corresponds to the pixel's position is True or 0,
            the pixel is calculation target.
            If you try to set array that values are all Falses or zeros, throws an exception.

        :Type:      numpy.array
        """
        if self._agc_roi_mask is None:
            self._agc_roi_mask = np.ones((self._frame_size[1], self._frame_size[0]), np.int)
        return self._agc_roi_mask

    @agc_roi_mask.setter
    def agc_roi_mask(self, mask):
        ow_set_agc_roi_mask(self._cobj, mask)
        self._agc_roi_mask = np.copy(mask)

    @property
    def window_correction(self):
        """
        .. only:: ja

            保護窓使用時の補正の種類。
            [書き込み可]

            ex) ow.window_correction = owlift.OwWindowCorrectionType.GAT05_STD_HOUSING

        .. only:: en

            Correction type for protection windows.
            [Writable]

            ex) ow.window_correction = owlift.OwWindowCorrectionType.GAT05_STD_HOUSING

        :Type:      :class:`OwWindowCorrectionType`
        """
        return self._window_correction

    @window_correction.setter
    def window_correction(self, wctype):
        if not isinstance(wctype, OwWindowCorrectionType):
            raise TypeError
        self._window_correction = wctype
        ow_set_window_correction(self._cobj, wctype.value, 0)

    def set_reflection_correction_file(self, file):
        """
        .. only:: ja

            反射補正データファイルを設定します。

            反射補正は保護窓に反射するOWLIFTの像を軽減します。
            反射補正データファイルは OWLIFTCap で生成してエクスポートしてください。

            * file (str)

                反射補正データファイルのパス。None を渡すと反射補正を解除します。

        .. only:: en

            Sets a reflection correction data file.

            Reflection correction reduce a reflected image of OWLIFT in a protection window.
            Reflection correction data file can be generated and exported by OWLIFTCap.

            * file (str)

                The path of the reflection correction data file.
                If None is passed, reflection correction is turned off.

        """
        if file is not None:
           ow_set_reflect_corr_file(self._cobj, file)
        else:
           ow_clear_reflect_corr_file(self._cobj)


    def enable_high_gain_mode(self):
        """
        .. only:: ja

            High Gain モードに切り替えます。
            Type-A では使用できません。

        .. only:: en

            Enables High Gain mode.
            Cannot use against Type-A.

        """
        ow_enable_high_gain_mode(self._cobj)

    def reconnect(self):
        """
        .. only:: ja

            ソフトウェア上でデバイスをホストから一旦切断し、再接続します。
            Type-F においてはファームウェアバージョン 3.1 以降で使用可能です。

        .. only:: en

            Disconnect the device from the host and reconnect it by software operation.
            Type-F supports this function only after the firmware version 3.1. 

        """
        ow_reconnect(self._cobj)

    def run_ffc(self):
        """
        .. only:: ja

            FFCを実行します。

        .. only:: en

            Execute FFC.

        """
        ow_run_ffc(self._cobj)

    def command_run(self, module_id, cmd_id_base):
        """
        .. only:: ja

            センサののコマンドをコマンドタイプ"RUN"で実行します。
            センサのコマンドについては `こちらのドキュメント <http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf>`_ を参照してください。
            再生中は失敗するときがあります。

            * module_id (int)

                Module ID.

            * cmd_id_base (int)

                Command ID Base.

        .. only:: en

            Executes the sensor's command as command type "RUN".
            See `this document <http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf>`_ about sensor's commands.
            An error possibly occurs after capture stared.

            * module_id (int)

                Module ID.

            * cmd_id_base (int)

                Command ID Base.

        """

        ow_command_run(self._cobj, module_id, cmd_id_base)

    def command_get(self, module_id, cmd_id_base, num_data, opt = 0):
        """
        .. only:: ja

            センサのコマンドをコマンドタイプ"GET"で実行し、結果のレジスタを読み込みます。
            センサのコマンドについては `こちらのドキュメント <http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf>`_ を参照してください。
            再生中は失敗するときがあります。

            * module_id (int)

                Module ID.

            * cmd_id_base (int)

                Command ID Base.

            * num_data

                 読み込むレジスタのWORD数。
                 読み込み可能なレジスタは DATA 0 Register .. DATA 15 Register であり、
                 ndata は DATA 0 Register を先頭として何個のレジスタを読み込むかを意味します。

            * opt

                使用不可。

            :Return:    numpy.array

         .. only:: en

            Executes the sensor's command as command type "GET", and reads registers as a result.
            See `this document <http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf>`_ about sensor's commands.
            An error possibly occurs after capture stared.

            * module_id (int)

                Module ID.

            * cmd_id_base (int)

                Command ID Base.

            * num_data

                The number of the registers to read in WORDs.
                DATA 0 Register .. DATA 15 Register are readable.
                'ndata' means the number of registers to read counting from DATA 0 Register.

            * opt

                This parameter cannot be used.

            :Return:    numpy.array
        """

        if num_data <= 0 or num_data > 16:
            raise ValueError('num_data')
        data_ba = ow_command_get(self._cobj, module_id, cmd_id_base, num_data, opt)
        return np.frombuffer(data_ba, dtype = np.uint16)

    def command_set(self, module_id, cmd_id_base, data):
        """
        .. only:: ja

            センサのレジスタに指定の値を書き込み、センサのコマンドをコマンドタイプ"SET"で実行します。
            センサのコマンドについては `こちらのドキュメント <http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf>`_ を参照してください。
            再生中は失敗するときがあります。

            * module_id (int)

                Module ID.

            * cmd_id_base (int)

                Command ID Base.

            * data (numpy.ndarray)

         		レジスタの値を示す numpy.uint16 型の配列。

        .. only:: en

            Writes the specified value to the register, and executes the sensor's command as command type "SET".
            See `this document <http://www.infinitegra.co.jp/static/owlift/docs/IDD.pdf>`_ about sensor's commands.
            An error possibly occurs after capture stared.

            * module_id (int)

                Module ID.

            * cmd_id_base (int)

                Command ID Base.

            * data (numpy.ndarray)

                 The array that stores registers' values of numpy.uint16.
        """

        if not isinstance(data, np.ndarray):
            raise TypeError('data')
        if len(data) == 0 or len(data) > 16:
            raise ValueError('data')
        ow_command_set(self._cobj, module_id, cmd_id_base, data.tobytes())

    def get_active(self):
        """
        .. only:: ja

            デバイスに再生中かどうかを問い合わせます。

            :Return: 再生中かどうか。

        .. only:: en

            Queries to the device whether it is active or not.

            :Return: Whether it is active or not.

        """

        return ow_get_active(self._cobj)
